#ifndef MZ_h
#define MZ_h

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the
//       Free Software Foundation, Inc.
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--


#include <algorithm>
#include <iomanip>
#include <iostream>
#include <map>
#include <string>

#include <bpipe/datatypes.h>

#include <paramxx/ParFile.h>
#include <suplib/debug.h>
#include <suplib/lists.h>
#include <suplib/str.h>
#include <tracefctxx/TraceFct.h>
#include <Exception/Exception.h>

#include <math.h>
#include <mathconst/mathconst.h>

#include "Random.h"
#include "cmp.h"


using namespace std;


#define ARCSEC2RAD(theta) ( (theta / 3600) * (M_PI / 180) )
#define RAD2ARCSEC(theta) ( (theta / M_PI) * 180 * 3600 )


class MZ {
 public:
    MZ( const string& id, const string& filename, double zmin, double zmax ) throw ( Exception ) :
    _id(id), _file( filename ), _zmin(zmin), _zmax(zmax) { }

  virtual ~MZ( void ) { };

  enum ScatterStatus {
    OK 			   = 0x0000,
    SCATTERED_INTO_SURFACE = 0x0001,
    BACKSCATTERED 	   = 0x0002,
    IP_EXTRAPOLATED 	   = 0x0004,
    OP_EXTRAPOLATED 	   = 0x0008
  };

  int scatter(
               bool in_plane, bool out_of_plane, int iter_limit,
	       double theta_def, double phi_def,
	       double min_cum_prob, double max_cum_prob,
	       const unsigned int id, const double energy,
	       const DVector3& surface_norm, DVector3& direction,
	       double& theta_o, double& phi_o
	       );

  virtual bool lookUpScatterAngle( double energy, double sin_alpha, double rannum,
				   bool inplane, double& theta ) = 0;
  double interpolate( double x, double xa, double ya, double xb, double yb );

  const string& id( void ) const { return _id; };
  long nRows( void ) const { return _nrows; };

  double zmin( void ) const { return _zmin; };
  double zmax( void ) const { return _zmax; };

  void zmin( const double d ) { _zmin = d; };
  void zmax( const double d ) { _zmax = d; };


 protected:

  string _id;
  string _file;
  double _zmin;
  double _zmax;

  long _nrows;

  struct Column2D  {

      size_t ncols;
      vector<double> data;

      double& operator ()( const long row, const long col )  { return data[ row * ncols + col ] ;}


  };



};


class mz_less : public less<MZ*> {
 public:
  bool operator() ( MZ* const& t1, MZ* const& t2 ) {
    return t1->zmin() < t2->zmax();
  };
};
#endif // #ifndef MZ_h
